/*
  Copyright (c) 2000,2001 Al Sutton (al@alsutton.com)
  All rights reserved.
  Redistribution and use in source and binary forms, with or without modification, are permitted
  provided that the following conditions are met:

  1. Redistributions of source code must retain the above copyright notice, this list of conditions
  and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright notice, this list of
  conditions and the following disclaimer in the documentation and/or other materials provided with
  the distribution.

  Neither the name of AlSutton.com nor the names of its contributors may be used to endorse or promote
  products derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package com.alsutton.filesync;

import java.io.*;
import java.net.*;

/**
 * Class invoked from the command line which determines the actions to be
 * performed.
 */
public class FileSync
{

  /**
   * The source for the synchronization
   */

  private String source;

  /**
   * The destination for the synchronization
   */

  private String destination;

  public FileSync( String _source, String _destination)
  {
    source = _source;
    destination = _destination;
  }


  public void synchronizeFiles( )
    throws IOException
  {
    if( source.equalsIgnoreCase( "-receiver" ) )
    {
      isReceivingServer( );
    }
    else if ( destination.indexOf( ':' ) != -1 )
    {
      isSendingToServer( );
    }
    else
    {
      isLocalTransfer( );
    }
  }

  private void isReceivingServer()
    throws IOException
  {
    System.out.print( "Listening on port 5200 ... ");
    ServerSocket socket = new ServerSocket( 5200 );
    Socket connection = socket.accept();

    String clientName = socket.getInetAddress().getHostName();
    System.out.println( "client connection from " + clientName );

    OutputStream toOutputStream = connection.getOutputStream();
    InputStream fromInputStream = connection.getInputStream();

    Receiver receiver = new Receiver( destination, fromInputStream, toOutputStream );
  }

  private void isSendingToServer( )
    throws IOException
  {
      File baseFile = new File( source );
      String realFilename = baseFile.getAbsolutePath();

      int colonIdx = destination.indexOf(':');
      String host = destination.substring(0, colonIdx);
      Socket socket = new Socket( host, 5200 );

      OutputStream fromOutputStream = socket.getOutputStream();
      InputStream toInputStream = socket.getInputStream();
      Synchronizer synchronizer = new Synchronizer( realFilename, toInputStream, fromOutputStream );

      synchronizer.sendData( "" );
      synchronizer.sendEndOfTransmittion();
  }

  private void isLocalTransfer( )
    throws IOException
  {
      File baseFile = new File( source );
      String realFilename = baseFile.getAbsolutePath();

      PipedOutputStream toOutputStream = new PipedOutputStream( );
      PipedInputStream fromInputStream = new PipedInputStream( );
      Receiver receiver = new Receiver( destination, fromInputStream, toOutputStream );

      PipedOutputStream fromOutputStream = new PipedOutputStream( fromInputStream );
      PipedInputStream toInputStream = new PipedInputStream( toOutputStream );
      Synchronizer synchronizer = new Synchronizer( realFilename, toInputStream, fromOutputStream );

      synchronizer.sendData( "" );
      synchronizer.sendEndOfTransmittion();
  }

  /**
   * The main method run by the user.
   */

  public static final void main( String[] args )
  {
    System.out.println( "FileSync, (c)Copyright 2001 Al Sutton, All Rights Reserved");
    System.out.println( "Please report any bugs to filesync-bugs@alsutton.com");
    System.out.println(  );

    if( args.length != 2 )
    {
      System.out.println( "Usage:" );

      System.out.println( "To copy locally: java -jar filesync.jar from_filename to_filename" );
      System.out.println( "To set up a receiver: java -jar filesync.jar -receiver to_filename" );
      System.out.println( "To send to another machine: java -jar filesync.jar from_filename server_name:" );
      System.out.println( "Note: The other machine must already be running a receiver." );     System.exit( -1 );
    }

    try
    {
      new FileSync( args[0], args[1] ).synchronizeFiles();
    }
    catch( Exception e )
    {
      e.printStackTrace();
    }

    System.out.println(  );
    System.out.println( "Program Finished" );
  }
}