/*
  Copyright (c) 2000,2001 Al Sutton (al@alsutton.com)
  All rights reserved.
  Redistribution and use in source and binary forms, with or without modification, are permitted
  provided that the following conditions are met:

  1. Redistributions of source code must retain the above copyright notice, this list of conditions
  and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright notice, this list of
  conditions and the following disclaimer in the documentation and/or other materials provided with
  the distribution.

  Neither the name of AlSutton.com nor the names of its contributors may be used to endorse or promote
  products derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package com.alsutton.filesync;

import java.io.*;

/**
 * Interface for all classes holding synchronization data.
 */

public abstract class Synchronizable
{
  /**
   * The filename relative to the base directory.
   */

  protected String filename;

  /**
   * Default constructor. Protected for internal use only.
   */

  protected Synchronizable()
  {
  }

  /**
   * Constructor. Extracts the name of the file from the File object
   * representation.
   *
   * @param file The java.io.File object representation.
   */

  public Synchronizable( String base, File theFile )
  {
    filename = theFile.getAbsolutePath();
    int sizeOfBase = base.length();
    filename = filename.substring( sizeOfBase );
  }

  /**
   * Method to write this data to a stream.
   *
   * @param stream The stream to write it to.
   */

  public abstract void write( OutputStream stream )
    throws IOException;

  /**
   * Constructor, Reconstructs the data from strea,.
   *
   * @param stream The stream holding the data.
   */

  public abstract void read( InputStream stream )
    throws IOException;

  /**
   * Method to compare this object to an object in the given base.
   *
   * @param newBase The new base to check the file against.
   * @return true if the file is the same, false otherwise.
   */

  public abstract boolean checkAgainstBase( String newBase );

  /**
   * Method to send the object content over a stream.
   *
   * @param base The base for the file to be retrieved.
   * @param stream The stream over which it should be sent.
   */

  public abstract void sendData( String base, OutputStream stream )
    throws IOException;

  /**
   * Method to the object contect from a stream.
   *
   * @param base The new base area to receive it into.
   * @param stream The stream over which to accept the file.
   */

  public abstract void receiveData( String base, InputStream stream )
    throws IOException;
}